import json
from pathlib import Path

__all__ = ["__version__"]

def _fetchVersion():
    HERE = Path(__file__).parent.resolve()

    for settings in HERE.rglob("package.json"): 
        try:
            with settings.open() as f:
                return json.load(f)["version"]
        except FileNotFoundError:
            pass

    raise FileNotFoundError(f"Could not find package.json under dir {HERE!s}")

__version__ = _fetchVersion()



from jupyter_server.base.handlers import APIHandler
from jupyter_server.utils import url_path_join
from tornado.escape import json_encode, json_decode
import tornado
import openai

class RouteHandler(APIHandler):
    # The following decorator should be present on all verb methods (head, get, post,
    # patch, put, delete, options) to ensure only authorized user can request the
    # Jupyter server
    @tornado.web.authenticated
    def post(self):
        if len(self.request.body) == 0:
            self.set_status(400)
            self.finish(json_encode({ "message": "no body provided" }))
            return

        data = json_decode(self.request.body)

        response = openai.Completion.create(
            api_key=data.get("api_key"),
            engine=data.get("engine", "davinci-codex"),
            prompt=data.get("prompt", ""),
            max_tokens=data.get("max_tokens", 256),
            temperature=data.get("temperature", 0.5),
            stop=data.get("stop", ["####"]),
        )

        self.finish(json_encode(response))


def setup_handlers(web_app):
    host_pattern = ".*$"

    base_url = web_app.settings["base_url"]
    route_pattern = url_path_join(base_url, "jupyterlab-codex", "completion")
    handlers = [(route_pattern, RouteHandler)]
    web_app.add_handlers(host_pattern, handlers)




import json
from pathlib import Path
from traitlets import Unicode
from traitlets.config import Configurable

from ._version import __version__

HERE = Path(__file__).parent.resolve()

with (HERE / "labextension" / "package.json").open() as fid:
    data = json.load(fid)

def _jupyter_labextension_paths():
    return [{
        "src": "labextension",
        "dest": data["name"]
    }]



from .handlers import setup_handlers


def _jupyter_server_extension_points():
    return [{
        "module": "jupyterlab_codex"
    }]


# # https://github.com/jupyterlab/jupyterlab-git/blob/65f5ed844780c87f34a8155b6ff21f41b6adf179/jupyterlab_git/__init__.py#L24
# class JupyterLabCodex(Configurable):
#     """
#     Config options for the JupyterLab codex extension.
#     """

#     api_key = Unicode(help="API key for the OpenAI API")

def _load_jupyter_server_extension(server_app):
    """Registers the API handler to receive HTTP requests from the frontend extension.

    Parameters
    ----------
    server_app: jupyterlab.labapp.LabApp
        JupyterLab application instance
    """
    # config = JupyterLabCodex(config=server_app.config)
    # server_app.web_app.settings["openai"] = config
    setup_handlers(server_app.web_app)
    server_app.log.info("Registered HelloWorld extension at URL path /jupyterlab-codex")

# For backward compatibility with notebook server - useful for Binder/JupyterHub
load_jupyter_server_extension = _load_jupyter_server_extension

